#include <S_Control_llenado_ultrasonico.h>
#include "internal_eeprom.c"

#FUSES NOMCLR

#define LCD_RS_PIN PIN_B0 // ASIGNAMOS EL PIN "B1" COMO SALIDA PARA RSW
#define LCD_RW_PIN PIN_B1 // ASIGNAMOS EL PIN "B2" COMO SALIDA PARA RW
#define LCD_ENABLE_PIN PIN_B2// ASIGNAMOS EL PIN "B0" COMO SALIDA PARA E
#define LCD_DATA4 PIN_B3 // ASIGNAMOS EL PIN "B3" COMO SALIDA PARA D4
#define LCD_DATA5 PIN_B4 // ASIGNAMOS EL PIN "B4" COMO SALIDA PARA D5
#define LCD_DATA6 PIN_B5 // ASIGNAMOS EL PIN "B5" COMO SALIDA PARA D6
#define LCD_DATA7 PIN_B6 // ASIGNAMOS EL PIN "B6" COMO SALIDA PARA D7
#include "lcd.c"

#define trig pin_D0 /*triger pin ultrasonico*/
#define echo pin_D1 /*echo pin ultrasnico*/
#define trig2 pin_D2 /*triger pin ultrasonico2*/
#define echo2 pin_D3 /*echo pin ultrasnico2*/

#define menu_ok input(pin_D4) /*botn de menu/ok*/
#define menos input(pin_D5)   /*botn de menos*/
#define mas input(pin_D6)     /*botn de ms*/

#define motor pin_B7          /*seal de salida para activar/desactivar motor*/

int16 tinaco = 0, cisterna = 0;
int16 altura_cisterna = 0, altura_tinaco = 0;
int8 llenado = 0, vaciado = 0;
int1 estado_motor = 0;

void sensor_cisterna();
void sensor_tinaco();
void mostrar();
void menu();
void cisterna_select();
void tinaco_select();
void llenado_select();
void vaciado_select();
void activar_motor();

void main()
{
   delay_ms(2000);
   
   lcd_init ();
   
   setup_timer_1(T1_INTERNAL|T1_DIV_BY_1);
   
   altura_cisterna=read_int16_eeprom(1);
   altura_tinaco=read_int16_eeprom(3);
   llenado = read_int16_eeprom(5);
   vaciado = read_int16_eeprom(7);
   
   if(altura_cisterna >= 400){altura_cisterna = 400;}
   if(altura_tinaco >= 400){altura_tinaco = 400;}
   if(llenado >= 100){llenado = 100;}
   if(vaciado >= 100){vaciado = 100;}
   
   mostrar();
   
   while(TRUE)
   {
      sensor_cisterna();
      
      sensor_tinaco();
      
      mostrar();
      
      activar_motor();
      
      while(menu_ok)
      {
         delay_ms(10);
         if(!menu_ok)
         {
            menu();
         }
      }
      delay_ms(100);
   }
}


void sensor_cisterna()
{
   int16 tiempo;
   int16 distancia;
   
   output_bit(trig,1); /* enviamos pulso de la seal*/ 
   delay_us(100); /* con 100us de ancho de pulso */
   output_bit(trig,0);
    
   while(!input(echo)) /* flanco de bajada */                  
   {}
   set_timer1(0); /* inicializa el conteo de tiempo para recibir la seal */  
   while(input(echo))   //flanco de subida                  
   {}
   
   tiempo=get_timer1(); /* cuando se recibe la seal guarda el tiempo que tardo en regresar*/ 
   
   distancia = ((float)tiempo*0.0172);/* hacemos una conversion para calcular la distancia*/
   
   if (distancia <= 20){distancia = 20;}
   if (distancia >= altura_cisterna){distancia = altura_cisterna;}
   
   cisterna = altura_cisterna*100 - distancia*100;
   cisterna = cisterna / (altura_cisterna - 20);
}


void sensor_tinaco()
{
   int16 tiempo;
   int16 distancia;
   
   output_bit(trig2, 1); /* enviamos pulso de la seal*/ 
   delay_us(100); /* con 100us de ancho de pulso */
   output_bit(trig2, 0);
   
   while(!input(echo2)) /* flanco de bajada */                  
   {}
   set_timer1(0); /* inicializa el conteo de tiempo para recibir la seal */  
   while(input(echo2))   //flanco de subida                  
   {}
   
   tiempo=get_timer1(); /* cuando se recibe la seal guarda el tiempo que tardo en regresar*/ 
   
   distancia=((int16)tiempo*0.0172);/* hacemos una conversion para calcular la distancia*/
   
   if (distancia <= 20){distancia = 20;}
   if (distancia >= altura_tinaco){distancia = altura_tinaco;}
   
   tinaco = altura_tinaco*100 - distancia*100;
   tinaco = tinaco / (altura_tinaco - 20);
}


void activar_motor()
{
   if(estado_motor)
   {
      if (cisterna <= 1 || tinaco >= 99) 
      {
         output_bit(motor, 0);   //desactivando la etapa de potencia / relevador;
         estado_motor = 0;
      }
   }else if (cisterna >= vaciado && tinaco <= llenado)
   {
      output_bit(motor, 1);   //activando la etapa de potencia / relevador;
      estado_motor = 1;
   } 
}


void mostrar()
{
   lcd_gotoxy (1, 1);printf(LCD_PUTC, "TINACO: %03lu%% ",tinaco);
   lcd_gotoxy (1, 2);printf(LCD_PUTC, "CISTERNA: %03lu%% ",cisterna);
}


void menu()
{ 
   int8 opcion = 3;
   
   while (true)
   {
      switch (opcion) 
      {     
         case 3:
            lcd_gotoxy (1, 1);printf(LCD_PUTC, "INICIO LLENADO< ");
            lcd_gotoxy (1, 2);printf(LCD_PUTC, "INICIO VACIADO  ");           
         break;
         
         case 2:
            lcd_gotoxy (1, 1);printf(LCD_PUTC, "INICIO LLENADO  ");
            lcd_gotoxy (1, 2);printf(LCD_PUTC, "INICIO VACIADO< "); 
         break; 
         
         case 1:
            lcd_gotoxy (1, 1);printf(LCD_PUTC, "ALTURA CISTERNA<");   
            lcd_gotoxy (1, 2);printf(LCD_PUTC, "ALTURA TINACO   ");           
         break;
         
         case 0:
            lcd_gotoxy (1, 1);printf(LCD_PUTC, "ALTURA CISTERNA ");   
            lcd_gotoxy (1, 2);printf(LCD_PUTC, "ALTURA TINACO<  ");           
         break;
      }
      
      while(mas)
      {
         delay_ms(10);
         if(!mas)
         {
           if(opcion == 3){break;}
           opcion++;
         }
      }
      
      while(menos)
      {
         delay_ms(10);
         if(!menos)
         {
           if(opcion == 0){break;}
           opcion--;         
         }
      }
      
      while(menu_ok)
      {
         delay_ms(10);
         if(!menu_ok)
         {
           lcd_gotoxy (1, 1);printf(LCD_PUTC, "\f");
           
           if (opcion == 3){llenado_select();}
           if (opcion == 2){vaciado_select();}
           if (opcion == 1){cisterna_select();}
           if (opcion == 0){tinaco_select();}
     
           return;
         }
      }
   } 
}


void cisterna_select()
{
   
   while(true)
   {
 
      lcd_gotoxy (1,1);printf(LCD_PUTC, "ALTURA CISTERNA");   
      lcd_gotoxy (1,2);printf(LCD_PUTC, "%03lucm", altura_cisterna);
      
      while(mas)
      {
         delay_ms(10);
         if(!mas)
         {
           if(altura_cisterna == 400){break;}
           altura_cisterna++;
         }
      }
      
      while(menos)
      {
         delay_ms(10);
         if(!menos)
         {
           if(altura_cisterna == 0){break;}
           altura_cisterna--;         
         }
      }
      
      write_int16_eeprom(1, altura_cisterna); // guarda el valor en la memoria EEPROM
      
      while(menu_ok)
      {
         delay_ms(10);
         if(!menu_ok)
         {
           lcd_gotoxy (1,1);printf(LCD_PUTC, "\f");
           return;
         }
      }
   }
}


void tinaco_select()
{
   
   while(true)
   {
 
      lcd_gotoxy (1,1);printf(LCD_PUTC, "ALTURA TINACO");   
      lcd_gotoxy (1,2);printf(LCD_PUTC, "%03lucm", altura_tinaco);
      
      while(mas)
      {
         delay_ms(10);
         if(!mas)
         {
           if(altura_tinaco == 400){break;}
           altura_tinaco++;
         }
      }
      
      while(menos)
      {
         delay_ms(10);
         if(!menos)
         {
           if(altura_tinaco == 0){break;}
           altura_tinaco--;         
         }
      }
      
      write_int16_eeprom(3, altura_tinaco);  // guarda el valor en la memoria EEPROM
      
      while(menu_ok)
      {
         delay_ms(10);
         if(!menu_ok)
         {
           lcd_gotoxy (1,1);printf(LCD_PUTC, "\f");
           return;
         }
      }
   }
}


void llenado_select()
{
   
   while(true)
   {
 
      lcd_gotoxy (1,1);printf(LCD_PUTC, "INICIO LLENADO");   
      lcd_gotoxy (1,2);printf(LCD_PUTC, "%03u%%", llenado);
      
      while(mas)
      {
         delay_ms(10);
         if(!mas)
         {
           if(llenado == 100){break;}
           llenado++;   // incrementa la variable
         }
      }
      
      while(menos)
      {
         delay_ms(10);
         if(!menos)
         {
           if(llenado == 0){break;}
           llenado--;   // decrementa la variable         
         }
      }
      
      write_int16_eeprom(5, llenado);  // guarda el valor en la memoria EEPROM
      
      while(menu_ok)
      {
         delay_ms(10);
         if(!menu_ok)
         {
           lcd_gotoxy (1,1);printf(LCD_PUTC, "\f");
           return;
         }
      }
   }
}


void vaciado_select()
{
   
   while(true)
   {
 
      lcd_gotoxy (1,1);printf(LCD_PUTC, "INICIO VACIADO");   
      lcd_gotoxy (1,2);printf(LCD_PUTC, "%03u%%", vaciado);
      
      while(mas)
      {
         delay_ms(10);
         if(!mas)
         {
           if(vaciado == 100){break;}
           vaciado++;   // incrementa la variable
         }
      }
      
      while(menos)
      {
         delay_ms(10);
         if(!menos)
         {
           if(vaciado == 0){break;}
           vaciado--;   // decrementa la variable        
         }
      }
      
      write_int16_eeprom(7, vaciado);  // guarda el valor en la memoria EEPROM
      
      while(menu_ok)
      {
         delay_ms(10);
         if(!menu_ok)
         {
           lcd_gotoxy (1,1);printf(LCD_PUTC, "\f");
           return;
         }
      }
   }
}
